library ieee;
use ieee.std_logic_1164.all;

use work.wf_package.all;

entity tb_urv_igloo2 is
end;

architecture arch of tb_urv_igloo2 is
  signal clk_40m : std_logic;
  signal devrst_n : std_logic;

  --  FIP
  signal fd_rxd, fd_txer, fd_wdgn, fd_rstn, fd_txd, fd_txena, fd_txck : std_logic;

  signal monimod_scl, monimod_sda : std_logic;

  signal leds : std_logic_vector(4 downto 1);

  signal s1_p_i  : std_logic_vector(15 downto 0);
  signal s1_n_o  : std_logic_vector(15 downto 0);
  signal s1_en_i : std_logic;
  signal s2_p_i  : std_logic_vector(15 downto 0); 
  signal s2_n_o  : std_logic_vector(15 downto 0); 
  signal s2_en_i : std_logic;
  signal s3_p_i  : std_logic_vector(15 downto 0);
  signal s3_n_o  : std_logic_vector(15 downto 0);
  signal s3_en_i : std_logic;
  signal s4_p_i  : std_logic_vector(15 downto 0);
  signal s4_n_o  : std_logic_vector(15 downto 0);
  signal s4_en_i : std_logic;
  signal s5_p_i  : std_logic_vector(15 downto 0);
  signal s5_n_o  : std_logic_vector(15 downto 0);
  signal s5_en_i : std_logic;
  signal s6_p_i  : std_logic_vector(15 downto 0);
  signal s6_n_o  : std_logic_vector(15 downto 0);
  signal s6_en_i : std_logic;
  signal s7_p_i  : std_logic_vector(15 downto 0);
  signal s7_n_o  : std_logic_vector(15 downto 0);
  signal s7_en_i : std_logic;
  signal s8_p_i  : std_logic_vector(15 downto 0);
  signal s8_n_o  : std_logic_vector(15 downto 0);
  signal s8_en_i : std_logic;

begin
  -- Clock (f=40Mhz, t=25ns)
  process
  begin
    clk_40m <= '0';
    wait for 12500 ps;
    clk_40m <= '1';
    wait for 12500 ps;
  end process;

  devrst_n <= '0', '1' after 100 ns;

  -- FIP tx
  process
    procedure wait_cycle is
    begin
      for i in 1 to 20 loop
        wait until rising_edge(clk_40m);
      end loop;
    end wait_cycle;

    procedure wait_cycles (n : natural) is
    begin
      for i in 1 to n loop
        wait_cycle;
      end loop;
    end wait_cycles;

    --  Send a manchester state (1 Mhz)
    procedure send_state (v : std_logic) is
    begin
      fd_rxd <= v;
      wait_cycle;
    end send_state;
    
    procedure send_bit (v : std_logic) is
      variable v2 : std_logic_vector(1 downto 0);
    begin
      if v = '1' then
        v2 := c_ONE;
      else
        v2 := c_ZERO;
      end if;
      send_state (v2(1));
      send_state (v2(0));
    end send_bit;

    procedure send_frame(fr : std_logic_vector)
    is
      variable crc : std_logic_vector (c_CRC_POLY_LGTH - 1 downto 0);
      variable crc0 : std_logic;
    begin
      for i in c_FSS'range loop
        send_state(c_FSS(i));
      end loop;

      crc := (others => '1');

      for i in fr'range loop
        --  Manchester
        send_bit (fr (i));

        crc0 := fr (i) xor crc(crc'left);
        for J in c_CRC_GENER_POLY'left downto 1 loop
          crc(J) := crc(J-1) xor (c_CRC_GENER_POLY(J) and crc0);
        end loop;
        crc (0) := crc0;
      end loop;

      --  Checksum (TODO)
      for i in crc'range loop
        send_bit (not crc (i));
      end loop;

      --  FES
      for i in c_FES'range loop
        send_state(c_FES(i));
      end loop;
    end send_frame;

    procedure send_id_dat(id : std_logic_vector(15 downto 0)) is
    begin
      send_frame("00" & c_ID_DAT_CTRL_BYTE & id);
    end send_id_dat;
  
    procedure send_rp_dat(dat : std_logic_vector) is
    begin
      assert dat'length mod 8 = 0;
      send_frame("00" & c_RP_DAT_CTRL_BYTE & dat);
    end send_rp_dat;
  begin
    fd_rxd <= '0';
    wait until clk_40m = '1';
    for i in 1 to 40 loop
      wait until rising_edge(clk_40m);
    end loop;

    --  ID dat for var1
    send_id_dat (x"05_01");
    wait_cycles(2);

    --  RP dat for var1
    send_rp_dat(c_PDU_TYPE_BYTE & x"08" & x"00_01_02_03_04_05_06_07");
    wait_cycles(2);

    send_id_dat (x"06_01");
    wait_cycles(2);

    wait;
  end process;

  fd_txer <= '0';
  fd_wdgn <= '0';

  --  Pull-up
  monimod_sda <= 'H';
  monimod_scl <= 'H';

  inst_top: entity work.diot_urv_top
    port map (
      clk_40m_i => clk_40m,
      devrst_n => devrst_n,
      fd_rxd_i => fd_rxd,
      fd_txer_i => fd_txer,
      fd_wdgn_i => fd_wdgn,
      fd_rstn_o => fd_rstn,
      fd_txck_o => fd_txck,
      fd_txd_o => fd_txd,
      fd_txena_o => fd_txena,
      monimod_scl_b => monimod_scl,
      monimod_sda_b => monimod_sda,

      mon_gpio_pres_i => "00",
      flash_miso_i => '0',
      
      s1_p_i => s1_p_i,
      s1_n_o => s1_n_o,
      s1_en_i => s1_en_i,
      s2_p_i => s2_p_i,
      s2_n_o => s2_n_o,
      s2_en_i => s2_en_i,
      s3_p_i => s3_p_i,
      s3_n_o => s3_n_o,
      s3_en_i => s3_en_i,
      s4_p_i => s4_p_i,
      s4_n_o => s4_n_o,
      s4_en_i => s4_en_i,
      s5_p_i => s5_p_i,
      s5_n_o => s5_n_o,
      s5_en_i => s5_en_i,
      s6_p_i => s6_p_i,
      s6_n_o => s6_n_o,
      s6_en_i => s6_en_i,
      s7_p_i => s7_p_i,
      s7_n_o => s7_n_o,
      s7_en_i => s7_en_i,
      s8_p_i => s8_p_i,
      s8_n_o => s8_n_o,
      s8_en_i => s8_en_i,
      leds_o => leds
    );
end arch;